
# q-expansion of newform 1232.4.a.j, downloaded from the LMFDB on 07 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1232
weight = 4
poly_data = [-9, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [-3, -1], [13, -1], [-7, 0], [-11, 0], [1, 1], [-44, -4], [-21, 19], [-80, -4], [12, 6], [-124, 14], [-26, 28], [264, 24], [102, -14], [-12, 90], [124, -46], [-285, -27], [-669, 23], [90, 34], [30, 150], [-2, -14], [964, 60], [271, -145], [-70, 208], [-1356, -82], [31, -241], [368, -154], [-482, -214], [322, -140], [1180, 134], [956, 32], [1507, 119], [-122, 128], [-191, -135], [880, -274], [792, -164], [-1441, -303], [1854, -74], [-2154, -330], [-3527, 137], [-2632, 124], [-705, 221], [-4326, 102], [-894, 500], [-1390, 616], [-2600, -426], [-2378, -270], [1418, -160], [-1053, -609], [3095, -339], [-1882, -176], [-5188, -32], [-648, 560], [2093, 283], [-1826, -844], [1452, -396], [-4217, 533], [-3812, 48], [3296, -162], [5422, 32], [-3357, 727], [-1261, 775], [567, -893], [-2690, 380], [2168, -42], [1170, -1284], [3664, 276], [4678, -68], [-3958, 1270], [3421, 361], [486, -300], [1572, 288], [3694, -708], [-6798, -832], [-1250, -930], [3596, 262], [6900, -378], [1371, -1099], [392, 2410], [4608, 812], [-4995, 927], [692, 1938], [9156, -576], [-5428, 534], [60, 568], [-320, -988], [-13958, 548], [-6678, -772], [-767, -1267], [2162, -1966], [-6981, 1641], [-990, -2202], [3796, 2592], [-4012, 376], [-2030, -3102], [3848, 364], [-1253, 257], [-2136, -702], [-11153, -1629], [4230, -1780], [-16712, 492], [4638, 804], [19141, -1147], [6434, 184], [-11868, -1112], [6302, -2376], [-11655, -441], [-25302, -282], [-5838, -2070], [-6498, 326], [-11856, 352], [17452, -1106], [20968, 1190], [-941, 129], [1254, -2894], [840, -2922], [-3203, -1893], [1592, 490], [-10512, 1794], [1122, -474], [-24395, 655], [-166, 780], [2861, -3155], [12168, 2172], [-11581, -1147], [14166, 1800], [34888, 250], [6168, -1590], [19076, -34], [-21397, -777], [676, 96], [-3572, 1736], [-18764, -1272], [-23128, 738], [-2052, 2064], [-15588, -2608], [13401, -4305], [6567, 3223], [-21909, 1965], [7874, 3220], [-11477, -369], [-9772, 298], [4786, -3258], [2324, -6032], [3777, 5027], [-9000, -2646], [-19103, 1681], [-1688, -4372], [-819, 4291], [4646, 5602], [38926, 1360], [20726, -1304], [3600, 3076], [12486, 3090], [22314, 1570], [22512, 2652], [-12752, 5856], [8148, 198], [2406, 746], [10863, -4629], [-17760, -5652], [17790, -636], [-4392, 3796], [32283, 2265], [3238, -964], [13164, 1410], [-28166, 3234], [-4833, -817], [12594, 2708], [-19205, -871], [866, 1630], [-24433, -2151], [21096, -4080], [458, -924], [1856, 3614], [32846, 1480], [-18932, 7464], [29718, 2292], [25232, 5348], [-20617, 279], [-15444, -6284], [-27783, 5265], [-10160, -926], [-13110, -6966], [16332, 1860], [6151, -1555], [-8374, -1272], [-28861, -7519], [7618, -884], [-7118, 3458], [-598, -5412], [10254, 690], [-14230, -5662], [-24489, 5385], [-9424, -860], [11242, 5720], [-71630, -944], [-10066, -9228], [18702, 8514], [-14868, -5808], [66848, -1250], [74610, -1118], [-31683, 4307], [-57026, 706], [4803, 3081], [26365, 8615], [-22376, -6354], [-56744, -580], [1662, 3720], [-22847, 3603], [24130, 5464], [49653, 6435], [1296, 7648], [-19485, 2805], [49548, -2808], [9020, -2862], [54888, 3376], [-9100, 4192], [-29054, 890], [-13852, 2710], [-25884, 8102], [47628, -5096], [8042, -9380], [44748, -5348], [52367, -3101], [-11546, 2152], [-27082, 4144], [28416, -7056], [-12384, 12940], [-4134, -1026], [-18742, -12648], [-25287, 3397], [19420, 2988], [-44230, -4328], [-90281, 1485], [37956, 4320], [-20020, 10158], [-42996, -1614], [-93654, -774], [7258, -76], [-390, -9822], [53743, -8025], [19626, -12644], [-3880, 14646], [47470, 2102], [37046, -2426], [77654, -4720], [78967, 6689], [63652, 7608], [-69076, 2044], [-22278, -15088], [-66150, 966], [-49454, -1510], [-35018, 4348], [-108449, -1165], [70446, -4902], [-3202, 1188], [39081, 13009], [-52923, 8031], [21568, -8120], [-17928, -2396], [69528, 996], [-6353, -11573], [-35657, 5167], [-3024, 294], [-93623, -5793], [-26740, -12878], [-53958, 9804], [-67396, 7400], [-9200, -5230], [15443, -7467], [-62422, 6242], [76314, -8956], [-156, 11512], [-59562, -6622], [21166, 8532], [45700, 2600], [3182, -5124], [81510, 3560], [-27121, 2287], [39372, 5028], [-18702, -12494], [53262, 13338], [73223, -8889], [-3877, -4405], [88632, 8196], [129786, 5180], [-19536, 8358], [-100684, 9818], [84242, -7454], [93352, -11194], [61503, 4821], [-27822, -5520], [-70317, -11133], [124524, 5834], [78529, 1613], [-59454, 10132], [-58673, -8035], [84885, 2997], [-13739, -21867], [-115912, 3036], [114374, 8920], [55356, -17252], [-17970, -11154], [-14814, 12922], [-88610, -2288], [-14868, 9984], [-132271, -3717], [-20988, 7260], [1100, 8658], [-41652, 16332], [-63732, 16110], [-76674, 17160], [-68140, 9872], [-38156, 8276], [82278, -1438], [13169, 8719], [-104908, -3908], [-7898, -2792], [-49710, 2028], [-111487, -10123], [56190, 5948], [-86305, -1589], [-87486, 6718], [123302, -8096], [-60424, 1494], [25178, -21598], [-60209, -12963], [2028, 27336], [60302, -6104], [45118, -11708], [43902, 12444], [-9206, 8394], [-60367, -8471], [61256, -5188], [-10041, -6555], [-54470, -1472], [35050, -21070], [-44234, -12500], [7782, 3610], [-86490, -13192], [-124608, -8538], [-7087, 19417], [90858, -3938], [-97980, -954], [-62300, -14776], [106323, 4145], [-115742, 10026], [-43108, 9484], [-116635, 16531], [-8459, 26853], [-93598, 7956], [114126, -17232], [-69658, 8978], [20332, 24058], [97946, -15974], [-12292, -23558], [-4297, 2563], [-96774, 10872], [1936, -13096], [171632, 126], [99134, 21838], [115708, -10324], [-165688, 11164], [-38861, 26385], [64736, -13218], [-18923, 9221], [-68908, 3944], [-19642, -4584], [42030, 20910], [62489, -15565], [18204, 33732], [-13050, 540], [-77392, -6688], [60884, -3606], [-104401, -4667], [105028, -7432], [-166354, 10638], [14792, 2176], [-99664, -6480], [88279, -20287], [80880, 822], [-44204, 3756], [31570, -14752], [68881, 13953], [1938, -17712], [-97938, -3350], [-131295, -4045], [-100990, -13834], [13776, 5142], [-18864, -18826], [-6795, 4683], [60730, -2974], [-108727, -13441], [95312, 6448], [-94678, 6948], [69242, -19906], [18056, -19730], [-142030, -3790], [-50442, 22944], [-183657, 851], [-111342, 28032], [-10740, 18100], [47864, -14858], [76349, -5099], [117010, 11274], [-136534, 14460], [-182634, 12552], [-140187, -8729], [108193, 29531], [-200050, 1818], [97808, 16954], [-184632, -3252], [21152, 30740], [78630, -20560], [106699, -1099], [-10996, -37724], [-55020, -18602], [98524, -3790], [45036, -31692], [56064, -18582], [-23417, 1243], [92630, -24410], [59505, -9843], [30992, 32262], [-9735, -9177], [-87858, -21954], [35806, -16600], [-75459, 22591], [-9006, 12726]]
